@@ -4,6 +4,7 @@
 #include "actionsmodel.h"
 
 #include "chatbarcache.h"
+#include "neochatconnection.h"
 #include "neochatroom.h"
 #include "roommanager.h"
 #include <Quotient/events/roommemberevent.h>
@@ -230,7 +231,7 @@ QList<ActionsModel::Action> actions{
             }
             auto targetRoom = text.startsWith(QLatin1Char('!')) ? room->connection()->room(text) : room->connection()->roomByAlias(text);
             if (targetRoom) {
-                RoomManager::instance().enterRoom(dynamic_cast<NeoChatRoom *>(targetRoom));
+                RoomManager::instance().resolveResource(targetRoom->id());
                 return QString();
             }
             Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("Joining room <roomname>.", "Joining room %1.", text));
@@ -256,11 +257,11 @@ QList<ActionsModel::Action> actions{
             }
             auto targetRoom = text.startsWith(QLatin1Char('!')) ? room->connection()->room(text) : room->connection()->roomByAlias(text);
             if (targetRoom) {
-                RoomManager::instance().enterRoom(dynamic_cast<NeoChatRoom *>(targetRoom));
+                RoomManager::instance().resolveResource(targetRoom->id());
                 return QString();
             }
             Q_EMIT room->showMessage(NeoChatRoom::Info, i18nc("Knocking room <roomname>.", "Knocking room %1.", text));
-            auto connection = room->connection();
+            auto connection = dynamic_cast<NeoChatConnection *>(room->connection());
             const auto knownServer = roomName.mid(roomName.indexOf(":"_ls) + 1);
             if (parts.length() >= 2) {
                 RoomManager::instance().knockRoom(connection, roomName, parts[1], QStringList{knownServer});
