@@ -36,6 +36,14 @@ MessageFilterModel::MessageFilterModel(QObject *parent, TimelineModel *sourceMod
 }
 
 bool MessageFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
+{
+    if (NeoChatConfig::self()->showAllEvents()) {
+        return true;
+    }
+    return eventIsVisible(sourceRow, sourceParent);
+}
+
+bool MessageFilterModel::eventIsVisible(int sourceRow, const QModelIndex &sourceParent) const
 {
     const QModelIndex index = sourceModel()->index(sourceRow, 0, sourceParent);
 
@@ -59,9 +67,8 @@ bool MessageFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sour
     // Don't show state events that are not the first in a consecutive group on the
     // same day as they will be grouped as a single delegate.
     const bool notLastRow = sourceRow < sourceModel()->rowCount() - 1;
-    const bool previousEventIsState = notLastRow
-        ? sourceModel()->data(sourceModel()->index(sourceRow + 1, 0), MessageEventModel::DelegateTypeRole) == DelegateType::State
-        : false;
+    const bool previousEventIsState =
+        notLastRow ? sourceModel()->data(sourceModel()->index(sourceRow + 1, 0), MessageEventModel::DelegateTypeRole) == DelegateType::State : false;
     const bool newDay = sourceModel()->data(sourceModel()->index(sourceRow, 0), MessageEventModel::ShowSectionRole).toBool();
     if (eventType == DelegateType::State && notLastRow && previousEventIsState && !newDay) {
         return false;
@@ -72,7 +79,11 @@ bool MessageFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sour
 
 QVariant MessageFilterModel::data(const QModelIndex &index, int role) const
 {
-    if (role == AggregateDisplayRole) {
+    if (role == MessageEventModel::DelegateTypeRole && NeoChatConfig::self()->showAllEvents()) {
+        if (!eventIsVisible(index.row(), index.parent())) {
+            return DelegateType::Other;
+        }
+    } else if (role == AggregateDisplayRole) {
         return aggregateEventToString(mapToSource(index).row());
     } else if (role == StateEventsRole) {
         return stateEventsList(mapToSource(index).row());
@@ -80,8 +91,24 @@ QVariant MessageFilterModel::data(const QModelIndex &index, int role) const
         return authorList(mapToSource(index).row());
     } else if (role == ExcessAuthorsRole) {
         return excessAuthors(mapToSource(index).row());
+    } else if (role == ShowAuthorRole) {
+        for (auto r = index.row() + 1; r < rowCount(); ++r) {
+            auto i = this->index(r, 0);
+            // Note !itemData(i).empty() is a check for instances where rows have been removed, e.g. when the read marker is moved.
+            // While the row is removed the subsequent row indexes are not changed so we need to skip over the removed index.
+            // See - https://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveRows
+            if (data(i, MessageEventModel::SpecialMarksRole) != EventStatus::Hidden && !itemData(i).empty()) {
+                return data(i, MessageEventModel::AuthorRole) != data(index, MessageEventModel::AuthorRole)
+                    || data(i, MessageEventModel::DelegateTypeRole) == DelegateType::State
+                    || data(i, MessageEventModel::TimeRole).toDateTime().msecsTo(data(index, MessageEventModel::TimeRole).toDateTime()) > 600000
+                    || data(i, MessageEventModel::TimeRole).toDateTime().toLocalTime().date().day()
+                    != data(index, MessageEventModel::TimeRole).toDateTime().toLocalTime().date().day();
+            }
+        }
+
+        return true;
     }
-    return sourceModel()->data(mapToSource(index), role);
+    return QSortFilterProxyModel::data(index, role);
 }
 
 QHash<int, QByteArray> MessageFilterModel::roleNames() const
@@ -91,6 +118,7 @@ QHash<int, QByteArray> MessageFilterModel::roleNames() const
     roles[StateEventsRole] = "stateEvents";
     roles[AuthorListRole] = "authorList";
     roles[ExcessAuthorsRole] = "excessAuthors";
+    roles[ShowAuthorRole] = "showAuthor";
     return roles;
 }
 
@@ -127,9 +155,8 @@ QString MessageFilterModel::aggregateEventToString(int sourceRow) const
         chunks.removeDuplicates();
         // The author text is either "n users" if > 1 user or the matrix.to link to a single user.
         QString userText = uniqueAuthors.length() > 1 ? i18ncp("n users", " %1 user ", " %1 users ", uniqueAuthors.length())
-                                                      : QStringLiteral("<a href=\"https://matrix.to/#/%1\" style=\"color: %2\">%3</a> ")
+                                                      : QStringLiteral("<a href=\"https://matrix.to/#/%1\">%3</a> ")
                                                             .arg(uniqueAuthors[0].toMap()[QStringLiteral("id")].toString(),
-                                                                 uniqueAuthors[0].toMap()[QStringLiteral("color")].toString(),
                                                                  uniqueAuthors[0].toMap()[QStringLiteral("displayName")].toString().toHtmlEscaped());
 
         QString chunksText;
