@@ -3,7 +3,6 @@
 
 #include "notificationsmodel.h"
 
-#include <Quotient/connection.h>
 #include <Quotient/events/event.h>
 #include <Quotient/uri.h>
 
@@ -122,9 +121,7 @@ void NotificationsModel::loadData()
                 const auto &authorAvatar = avatar.isValid() && avatar.scheme() == QStringLiteral("mxc") ? avatar : QUrl();
 
                 const auto &roomEvent = eventCast<const RoomEvent>(notification.event.get());
-                EventHandler eventHandler;
-                eventHandler.setRoom(dynamic_cast<NeoChatRoom *>(room));
-                eventHandler.setEvent(roomEvent);
+                EventHandler eventHandler(dynamic_cast<NeoChatRoom *>(room), roomEvent);
                 beginInsertRows({}, m_notifications.length(), m_notifications.length());
                 m_notifications += Notification{
                     .roomId = notification.roomId,
@@ -164,3 +161,5 @@ QString NotificationsModel::nextToken() const
 {
     return m_nextToken;
 }
+
+#include "moc_notificationsmodel.cpp"
