@@ -4,12 +4,20 @@
 #include "neochatconnection.h"
 
 #include <QImageReader>
+#include <QJsonDocument>
 
 #include "controller.h"
 #include "jobs/neochatchangepasswordjob.h"
 #include "jobs/neochatdeactivateaccountjob.h"
+#include "linkpreviewer.h"
+#include "neochatconfig.h"
+#include "neochatroom.h"
 #include "roommanager.h"
+#include "spacehierarchycache.h"
 
+#include <Quotient/connection.h>
+#include <Quotient/jobs/basejob.h>
+#include <Quotient/quotient_common.h>
 #include <qt6keychain/keychain.h>
 
 #include <KLocalizedString>
@@ -17,7 +25,9 @@
 #include <Quotient/csapi/content-repo.h>
 #include <Quotient/csapi/profile.h>
 #include <Quotient/database.h>
+#include <Quotient/jobs/downloadfilejob.h>
 #include <Quotient/qt_connection_util.h>
+#include <Quotient/room.h>
 #include <Quotient/settings.h>
 #include <Quotient/user.h>
 
@@ -32,6 +42,19 @@ using namespace Qt::StringLiterals;
 
 NeoChatConnection::NeoChatConnection(QObject *parent)
     : Connection(parent)
+    , m_threePIdModel(new ThreePIdModel(this))
+{
+    connectSignals();
+}
+
+NeoChatConnection::NeoChatConnection(const QUrl &server, QObject *parent)
+    : Connection(server, parent)
+    , m_threePIdModel(new ThreePIdModel(this))
+{
+    connectSignals();
+}
+
+void NeoChatConnection::connectSignals()
 {
     connect(this, &NeoChatConnection::accountDataChanged, this, [this](const QString &type) {
         if (type == QLatin1String("org.kde.neochat.account_label")) {
@@ -49,16 +72,78 @@ NeoChatConnection::NeoChatConnection(QObject *parent)
             Q_EMIT userConsentRequired(job->errorUrl());
         }
     });
-}
-
-NeoChatConnection::NeoChatConnection(const QUrl &server, QObject *parent)
-    : Connection(server, parent)
-{
-    connect(this, &NeoChatConnection::accountDataChanged, this, [this](const QString &type) {
-        if (type == QLatin1String("org.kde.neochat.account_label")) {
-            Q_EMIT labelChanged();
+    connect(this, &NeoChatConnection::requestFailed, this, [](BaseJob *job) {
+        if (dynamic_cast<DownloadFileJob *>(job) && job->jsonData()["errcode"_ls].toString() == "M_TOO_LARGE"_ls) {
+            RoomManager::instance().warning(i18n("File too large to download."), i18n("Contact your matrix server administrator for support."));
         }
     });
+    connect(this, &NeoChatConnection::directChatsListChanged, this, [this](DirectChatsMap additions, DirectChatsMap removals) {
+        Q_EMIT directChatInvitesChanged();
+        for (const auto &chatId : additions) {
+            if (const auto chat = room(chatId)) {
+                connect(chat, &Room::unreadStatsChanged, this, [this]() {
+                    refreshBadgeNotificationCount();
+                    Q_EMIT directChatNotificationsChanged();
+                    Q_EMIT directChatsHaveHighlightNotificationsChanged();
+                });
+            }
+        }
+        for (const auto &chatId : removals) {
+            if (const auto chat = room(chatId)) {
+                disconnect(chat, &Room::unreadStatsChanged, this, nullptr);
+            }
+        }
+    });
+    connect(this, &NeoChatConnection::joinedRoom, this, [this](Room *room) {
+        if (room->isDirectChat()) {
+            connect(room, &Room::unreadStatsChanged, this, [this]() {
+                Q_EMIT directChatNotificationsChanged();
+                Q_EMIT directChatsHaveHighlightNotificationsChanged();
+            });
+        }
+        connect(room, &Room::unreadStatsChanged, this, [this]() {
+            refreshBadgeNotificationCount();
+            Q_EMIT homeNotificationsChanged();
+            Q_EMIT homeHaveHighlightNotificationsChanged();
+        });
+    });
+    connect(this, &NeoChatConnection::leftRoom, this, [this](Room *room, Room *prev) {
+        Q_UNUSED(room)
+        if (prev && prev->isDirectChat()) {
+            Q_EMIT directChatInvitesChanged();
+            Q_EMIT directChatNotificationsChanged();
+            Q_EMIT directChatsHaveHighlightNotificationsChanged();
+        }
+        refreshBadgeNotificationCount();
+        Q_EMIT homeNotificationsChanged();
+        Q_EMIT homeHaveHighlightNotificationsChanged();
+    });
+
+    connect(&SpaceHierarchyCache::instance(), &SpaceHierarchyCache::spaceHierarchyChanged, this, [this]() {
+        refreshBadgeNotificationCount();
+        Q_EMIT homeNotificationsChanged();
+        Q_EMIT homeHaveHighlightNotificationsChanged();
+    });
+}
+
+int NeoChatConnection::badgeNotificationCount() const
+{
+    return m_badgeNotificationCount;
+}
+
+void NeoChatConnection::refreshBadgeNotificationCount()
+{
+    int count = 0;
+    for (const auto &r : allRooms()) {
+        if (const auto room = static_cast<NeoChatRoom *>(r)) {
+            count += room->contextAwareNotificationCount();
+        }
+    }
+
+    if (count != m_badgeNotificationCount) {
+        m_badgeNotificationCount = count;
+        Q_EMIT badgeNotificationCountChanged(this, m_badgeNotificationCount);
+    }
 }
 
 void NeoChatConnection::logout(bool serverSideLogout)
@@ -73,14 +158,6 @@ void NeoChatConnection::logout(bool serverSideLogout)
     job.start();
     loop.exec();
 
-    if (Controller::instance().accounts().count() > 1) {
-        // Only set the connection if the the account being logged out is currently active
-        if (this == Controller::instance().activeConnection()) {
-            Controller::instance().setActiveConnection(dynamic_cast<NeoChatConnection *>(Controller::instance().accounts().accounts()[0]));
-        }
-    } else {
-        Controller::instance().setActiveConnection(nullptr);
-    }
     if (!serverSideLogout) {
         return;
     }
@@ -174,6 +251,11 @@ void NeoChatConnection::deactivateAccount(const QString &password)
     });
 }
 
+ThreePIdModel *NeoChatConnection::threePIdModel() const
+{
+    return m_threePIdModel;
+}
+
 void NeoChatConnection::createRoom(const QString &name, const QString &topic, const QString &parent, bool setChildParent)
 {
     QList<CreateRoomJob::StateEvent> initialStateEvents;
@@ -202,7 +284,7 @@ void NeoChatConnection::createRoom(const QString &name, const QString &topic, co
         Q_EMIT Controller::instance().errorOccured(i18n("Room creation failed: %1", job->errorString()), {});
     });
     connectSingleShot(this, &Connection::newRoom, this, [](Room *room) {
-        RoomManager::instance().enterRoom(dynamic_cast<NeoChatRoom *>(room));
+        RoomManager::instance().resolveResource(room->id());
     });
 }
 
@@ -234,22 +316,106 @@ void NeoChatConnection::createSpace(const QString &name, const QString &topic, c
         Q_EMIT Controller::instance().errorOccured(i18n("Space creation failed: %1", job->errorString()), {});
     });
     connectSingleShot(this, &Connection::newRoom, this, [](Room *room) {
-        RoomManager::instance().enterRoom(dynamic_cast<NeoChatRoom *>(room));
+        RoomManager::instance().resolveResource(room->id());
     });
 }
 
+bool NeoChatConnection::directChatExists(Quotient::User *user)
+{
+    return directChats().contains(user);
+}
+
+void NeoChatConnection::openOrCreateDirectChat(const QString &userId)
+{
+    if (auto user = this->user(userId)) {
+        openOrCreateDirectChat(user);
+    } else {
+        qWarning() << "openOrCreateDirectChat: Couldn't get user object for ID " << userId << ", unable to open/request direct chat.";
+    }
+}
+
 void NeoChatConnection::openOrCreateDirectChat(User *user)
 {
     const auto existing = directChats();
 
     if (existing.contains(user)) {
-        const auto room = static_cast<NeoChatRoom *>(this->room(existing.value(user)));
+        const auto room = this->room(existing.value(user));
         if (room) {
-            RoomManager::instance().enterRoom(room);
+            RoomManager::instance().resolveResource(room->id());
             return;
         }
     }
     requestDirectChat(user);
+    connectSingleShot(this, &Connection::directChatAvailable, this, [=](auto room) {
+        room->activateEncryption();
+    });
+}
+
+qsizetype NeoChatConnection::directChatNotifications() const
+{
+    qsizetype notifications = 0;
+    QStringList added; // The same ID can be in the list multiple times.
+    for (const auto &chatId : directChats()) {
+        if (!added.contains(chatId)) {
+            if (const auto chat = room(chatId)) {
+                notifications += dynamic_cast<NeoChatRoom *>(chat)->contextAwareNotificationCount();
+                added += chatId;
+            }
+        }
+    }
+    return notifications;
+}
+
+bool NeoChatConnection::directChatsHaveHighlightNotifications() const
+{
+    for (const auto &childId : directChats()) {
+        if (const auto child = static_cast<NeoChatRoom *>(room(childId))) {
+            if (child->highlightCount() > 0) {
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
+qsizetype NeoChatConnection::homeNotifications() const
+{
+    qsizetype notifications = 0;
+    QStringList added;
+    const auto &spaceHierarchyCache = SpaceHierarchyCache::instance();
+    for (const auto &r : allRooms()) {
+        if (const auto room = static_cast<NeoChatRoom *>(r)) {
+            if (!added.contains(room->id()) && !room->isDirectChat() && !spaceHierarchyCache.isChild(room->id())) {
+                notifications += dynamic_cast<NeoChatRoom *>(room)->contextAwareNotificationCount();
+                added += room->id();
+            }
+        }
+    }
+    return notifications;
+}
+
+bool NeoChatConnection::homeHaveHighlightNotifications() const
+{
+    const auto &spaceHierarchyCache = SpaceHierarchyCache::instance();
+    for (const auto &r : allRooms()) {
+        if (const auto room = static_cast<NeoChatRoom *>(r)) {
+            if (!room->isDirectChat() && !spaceHierarchyCache.isChild(room->id()) && room->highlightCount() > 0) {
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
+bool NeoChatConnection::directChatInvites() const
+{
+    auto inviteRooms = rooms(JoinState::Invite);
+    for (const auto inviteRoom : inviteRooms) {
+        if (inviteRoom->isDirectChat()) {
+            return true;
+        }
+    }
+    return false;
 }
 
 QCoro::Task<void> NeoChatConnection::setupPushNotifications(QString endpoint)
@@ -316,4 +482,25 @@ void NeoChatConnection::setIsOnline(bool isOnline)
     Q_EMIT isOnlineChanged();
 }
 
+QString NeoChatConnection::accountDataJsonString(const QString &type) const
+{
+    return QString::fromUtf8(QJsonDocument(accountDataJson(type)).toJson());
+}
+
+LinkPreviewer *NeoChatConnection::previewerForLink(const QUrl &link)
+{
+    if (!NeoChatConfig::showLinkPreview()) {
+        return nullptr;
+    }
+
+    auto previewer = m_linkPreviewers.value(link, nullptr);
+    if (previewer != nullptr) {
+        return previewer;
+    }
+
+    previewer = new LinkPreviewer(link, this);
+    m_linkPreviewers[link] = previewer;
+    return previewer;
+}
+
 #include "moc_neochatconnection.cpp"
