@@ -11,20 +11,20 @@
 #include <Quotient/uriresolver.h>
 
 #include "chatdocumenthandler.h"
-#include "enums/delegatetype.h"
+#include "enums/messagecomponenttype.h"
+#include "eventhandler.h"
 #include "models/mediamessagefiltermodel.h"
 #include "models/messagefiltermodel.h"
+#include "models/roomlistmodel.h"
+#include "models/roomtreemodel.h"
+#include "models/sortfilterroomlistmodel.h"
+#include "models/sortfilterroomtreemodel.h"
+#include "models/sortfilterspacelistmodel.h"
 #include "models/timelinemodel.h"
 
 class NeoChatRoom;
 class NeoChatConnection;
 
-namespace Quotient
-{
-class Room;
-class User;
-}
-
 using namespace Quotient;
 
 /**
@@ -43,8 +43,6 @@ class RoomManager : public QObject, public UriResolverBase
     QML_ELEMENT
     QML_SINGLETON
 
-    Q_PROPERTY(NeoChatConnection *connection READ connection WRITE setConnection NOTIFY connectionChanged)
-
     /**
      * @brief The current open room in NeoChat, if any.
      *
@@ -52,6 +50,47 @@ class RoomManager : public QObject, public UriResolverBase
      */
     Q_PROPERTY(NeoChatRoom *currentRoom READ currentRoom NOTIFY currentRoomChanged)
 
+    /**
+     * @brief The id of the space currently opened in the space drawer.
+     *
+     * If this is an empty string, the uncategorized rooms are shown.
+     * If it is the string "DM", the DMs are shown.
+     */
+    Q_PROPERTY(QString currentSpace READ currentSpace WRITE setCurrentSpace NOTIFY currentSpaceChanged)
+
+    /**
+     * @brief The RoomListModel that should be used for linear room visualisation.
+     *
+     * The connection the model uses to get the data will be updated by this class
+     * so there is no need to do this manually or replace the model when the connection
+     * changes.
+     */
+    Q_PROPERTY(RoomListModel *roomListModel READ roomListModel CONSTANT)
+
+    /**
+     * @brief The SortFilterRoomListModel that should be used for room visualisation.
+     */
+    Q_PROPERTY(SortFilterRoomListModel *sortFilterRoomListModel READ sortFilterRoomListModel CONSTANT)
+
+    /**
+     * @brief The SortFilterSpaceListModel that should be used for space visualisation.
+     */
+    Q_PROPERTY(SortFilterSpaceListModel *sortFilterSpaceListModel READ sortFilterSpaceListModel CONSTANT)
+
+    /**
+     * @brief The RoomTreeModel that should be used for room visualisation.
+     *
+     * The connection the model uses to get the data will be updated by this class
+     * so there is no need to do this manually or replace the model when the connection
+     * changes.
+     */
+    Q_PROPERTY(RoomTreeModel *roomTreeModel READ roomTreeModel CONSTANT)
+
+    /**
+     * @brief The SortFilterRoomTreeModel that should be used for room visualisation.
+     */
+    Q_PROPERTY(SortFilterRoomTreeModel *sortFilterRoomTreeModel READ sortFilterRoomTreeModel CONSTANT)
+
     /**
      * @brief The TimelineModel that should be used for room message visualisation.
      *
@@ -105,6 +144,12 @@ public:
 
     NeoChatRoom *currentRoom() const;
 
+    RoomListModel *roomListModel() const;
+    SortFilterRoomListModel *sortFilterRoomListModel() const;
+    SortFilterSpaceListModel *sortFilterSpaceListModel() const;
+    RoomTreeModel *roomTreeModel() const;
+    SortFilterRoomTreeModel *sortFilterRoomTreeModel() const;
+
     TimelineModel *timelineModel() const;
     MessageFilterModel *messageFilterModel() const;
     MediaMessageFilterModel *mediaMessageFilterModel() const;
@@ -136,39 +181,18 @@ public:
      */
     Q_INVOKABLE void loadInitialRoom();
 
-    /**
-     * @brief Enter the given room.
-     *
-     * This method will tell NeoChat to open the message list with the given room.
-     */
-    Q_INVOKABLE void enterRoom(NeoChatRoom *room);
-
-    /**
-     * @brief Open a new window with the given room.
-     *
-     * The open window will have its own message list for the given room.
-     */
-    Q_INVOKABLE void openWindow(NeoChatRoom *room);
-
     /**
      * @brief Leave the room and close it if it is open.
      */
     Q_INVOKABLE void leaveRoom(NeoChatRoom *room);
 
-    /**
-     * @brief Enter the home page of the given space.
-     *
-     * This method will tell NeoChat to open the home page for the given space.
-     */
-    Q_INVOKABLE void enterSpaceHome(NeoChatRoom *spaceRoom);
-
     /**
      * @brief Knock a room.
      *
      * See https://spec.matrix.org/latest/client-server-api/#knocking-on-rooms for
      * knocking on rooms.
      */
-    void knockRoom(Quotient::Connection *account, const QString &roomAliasOrId, const QString &reason, const QStringList &viaServers);
+    void knockRoom(NeoChatConnection *account, const QString &roomAliasOrId, const QString &reason, const QStringList &viaServers);
 
     /**
      * @brief Show a media item maximized.
@@ -179,6 +203,8 @@ public:
      */
     Q_INVOKABLE void maximizeMedia(int index);
 
+    Q_INVOKABLE void maximizeCode(const QVariantMap &author, const QDateTime &time, const QString &codeText, const QString &language);
+
     /**
      * @brief Request that any full screen overlay currently open closes.
      */
@@ -190,21 +216,9 @@ public:
     Q_INVOKABLE void viewEventSource(const QString &eventId);
 
     /**
-     * @brief Show a conterxt menu for the given event.
+     * @brief Show a context menu for the given event.
      */
-    Q_INVOKABLE void viewEventMenu(const QString &eventId,
-                                   const QVariantMap &author,
-                                   DelegateType::Type delegateType,
-                                   const QString &plainText,
-                                   const QString &htmlText = {},
-                                   const QString &selectedText = {},
-                                   const QString &mimeType = {},
-                                   const FileTransferInfo &progressInfo = {});
-
-    /**
-     * @brief Call this when the current used connection is dropped.
-     */
-    Q_INVOKABLE void reset();
+    Q_INVOKABLE void viewEventMenu(const QString &eventId, NeoChatRoom *room, const QString &selectedText = {});
 
     ChatDocumentHandler *chatDocumentHandler() const;
     void setChatDocumentHandler(ChatDocumentHandler *handler);
@@ -214,65 +228,32 @@ public:
      */
     void setUrlArgument(const QString &arg);
 
-    NeoChatConnection *connection() const;
+    QString currentSpace() const;
+
+    bool directChatsActive() const;
+    void setDirectChatsActive(bool directChatsActive);
+
+    /**
+     * @brief Set the current connection
+     */
     void setConnection(NeoChatConnection *connection);
 
+    /**
+     * Closes the current room and space; for situations, where it is unclear which room should be opened.
+     */
+    void resetState();
+
 Q_SIGNALS:
+    /** Ask the user whether the room should be joined. */
+    void askJoinRoom(const QString &nameOrId);
+
     void currentRoomChanged();
 
-    /**
-     * @brief Push a new room page.
-     *
-     * Signal triggered when the main window pageStack should push a new page with
-     * the message list for the given room.
-     *
-     * @param room the room to be shown on the new page.
-     * @param event the event to got to if available.
-     */
-    void pushRoom(NeoChatRoom *room, const QString &event);
-
-    /**
-     * @brief Replace the existing room.
-     *
-     * Signal triggered when the room displayed by the message list should be changed.
-     *
-     * @param room the room to be shown on the new page.
-     * @param event the event to got to if available.
-     */
-    void replaceRoom(NeoChatRoom *room, const QString &event);
-
-    /**
-     * @brief Push a new space home page.
-     *
-     * Signal triggered when the main window pageStack should push a new page with
-     * the space home for the given space room.
-     *
-     * @param spaceRoom the space room to be shown on the new page.
-     */
-    void pushSpaceHome(NeoChatRoom *spaceRoom);
-
-    /**
-     * @brief Replace the existing space home.
-     *
-     * Signal triggered when the currently displayed room page should be changed
-     * to the space home for the given space room.
-     *
-     * @param spaceRoom the space room to be shown on the new page.
-     */
-    void replaceSpaceHome(NeoChatRoom *spaceRoom);
-
     /**
      * @brief Go to the specified event in the current room.
      */
     void goToEvent(const QString &event);
 
-    /**
-     * @brief Open room in a new window.
-     *
-     * Signal triggered when a room needs to be opened in a new window.
-     */
-    void openRoomInNewWindow(NeoChatRoom *room);
-
     /**
      * @brief Show details for the given user.
      *
@@ -290,6 +271,11 @@ Q_SIGNALS:
      */
     void showMaximizedMedia(int index);
 
+    /**
+     * @brief Request a block of code is shown maximized.
+     */
+    void showMaximizedCode(const QVariantMap &author, const QDateTime &time, const QString &codeText, const QString &language);
+
     /**
      * @brief Request that any full screen overlay closes.
      */
@@ -305,7 +291,7 @@ Q_SIGNALS:
      */
     void showMessageMenu(const QString &eventId,
                          const QVariantMap &author,
-                         DelegateType::Type delegateType,
+                         MessageComponentType::Type messageComponentType,
                          const QString &plainText,
                          const QString &htmlText,
                          const QString &selectedText);
@@ -315,7 +301,7 @@ Q_SIGNALS:
      */
     void showFileMenu(const QString &eventId,
                       const QVariantMap &author,
-                      DelegateType::Type delegateType,
+                      MessageComponentType::Type messageComponentType,
                       const QString &plainText,
                       const QString &mimeType,
                       const FileTransferInfo &progressInfo);
@@ -337,20 +323,47 @@ Q_SIGNALS:
 
     void connectionChanged();
 
+    void directChatsActiveChanged();
+
+    void externalUrl(const QUrl &url);
+
+    void currentSpaceChanged();
+
 private:
     void openRoomForActiveConnection();
 
-    NeoChatRoom *m_currentRoom;
-    NeoChatRoom *m_lastCurrentRoom;
+    /** The room currently being shown in the main view (RoomPage.qml). This can be null, if there is no room.
+     * If this is a space, the space home page is shown.
+     */
+    QPointer<NeoChatRoom> m_currentRoom;
+
+    /** The id of the space currently opened in the space drawer. If this is empty, the uncategorized rooms are shown.
+     * If it is "DM", the direct messages are shown. Otherwise it's the id of a toplevel space.
+     */
+    QString m_currentSpaceId;
+
     QString m_arg;
     KSharedConfig::Ptr m_config;
     KConfigGroup m_lastRoomConfig;
+    KConfigGroup m_lastSpaceConfig;
+    KConfigGroup m_directChatsConfig;
     QPointer<ChatDocumentHandler> m_chatDocumentHandler;
 
+    RoomListModel *m_roomListModel;
+    SortFilterRoomListModel *m_sortFilterRoomListModel;
+    SortFilterSpaceListModel *m_sortFilterSpaceListModel;
+    RoomTreeModel *m_roomTreeModel;
+    SortFilterRoomTreeModel *m_sortFilterRoomTreeModel;
+
     TimelineModel *m_timelineModel;
     MessageFilterModel *m_messageFilterModel;
     MediaMessageFilterModel *m_mediaMessageFilterModel;
-    NeoChatConnection *m_connection;
+    QPointer<NeoChatConnection> m_connection;
+
+    void setCurrentRoom(const QString &roomId);
+
+    // Space ID, "DM", or empty string
+    void setCurrentSpace(const QString &spaceId, bool setRoom = true);
 
     /**
      * @brief Resolve a user URI.
